from sisense.resource import Resource


class Permission(Resource):

    def get(self, elasticube: str) -> list:
        """Get permissions (shares) for the specified elasticube.

        :param elasticube: (str) Elasticube's name.
        :return: a list of permission objects
        """
        content = self._api.get(f'elasticubes/localhost/{elasticube}/permissions')

        permissions = []
        for p in content['shares']:
            p['party'] = p['partyId']
            del p['partyId']

            permissions.append(Permission(self._api, p))

        return permissions

    def add(self, elasticube: str, oid: str = None, ptype: str = None, level: str = None, permissions: list = None):
        """Add new permissions to the specified elasticube.
        If permissions is none and any other optional parameters are None, add the current permission.
        Use permissions parameter to add several permissions at once.
        Use the other parameters to add just one permission.

        :param elasticube: (str, default None) Elasticube's name.
        :param oid: (str, default None) Group/user id.
        :param ptype: (str, default None) Party's type. Possible values are: 'group' or 'user'.
        :param level: (str, default None) Permission's type. Possible values are: 'r' (read) or 'w' (write).
        :param permissions: (list) List of permissions JSONs to add all at once.
        """
        if permissions is None:
            if oid is None or type is None or level is None:
                data = [self._json]
            else:
                data = [{'party': oid, 'type': ptype, 'permission': level}]
        else:
            data = permissions

        all_permissions = self.get(elasticube)
        all_permissions = [p.json for p in all_permissions] + data

        self._api.put(f'elasticubes/localhost/{elasticube}/permissions', data=all_permissions)

    def delete_all(self, elasticube: str):
        """Delete all elasticube's permissions.

        :param elasticube: (str, default None) Elasticube's name.
        """
        self._api.delete(f'elasticubes/localhost/{elasticube}/permissions')



from .test_case import TestCaseV1 as TestCase


class UserTestCase(TestCase):

    def test_get(self):
        user = self.sisense.user.get(self.config['user_email'])
        self.assertEqual(user.email, self.config['user_email'])



from sisense.resource import Resource


class Group(Resource):

    def get(self, name: str) -> object:
        """Get group by name.

        :param name: (str) Group's name.
        :return: Group, if found. None, otherwise.
        """
        content = self._api.get('groups', query={'name': name})
        return Group(self._api, content[0]) if len(content) else None



from .test_case import TestCaseV1 as TestCase


class GroupTestCase(TestCase):

    def test_get(self):
        group = self.sisense.group.get(self.config['group_name'])
        self.assertEqual(group.name, self.config['group_name'])



from .api import API
import json


class Resource:

    def __init__(self, api: API, rjson: dict = None):
        """
        Super for any API's resource.

        :param api: (API) Used to make API's requests.
        :param rjson: (dict) Resource representation.
        """
        # If you add a new attribute, always remember to include it on __setattr__
        self._api = api
        self.json = rjson if rjson else {}

    def new(self, rjson: dict) -> object:
        """Create a new resource with different properties, but same API.

        :param rjson: (dict) Resource representation.
        :return: (Resource) The new resource
        """
        return Resource(self._api, rjson)

    def save(self, filepath: str):
        """
        Save the object representation as a .json file.

        :param filepath: (str) Relative/absolute path, including filename.
        """
        filepath = filepath if filepath.endswith('.json') else f'{filepath}.json'
        with open(filepath, 'r') as file:
            json.dump(self.json, file)

    def __setattr__(self, key, value):
        if key in self.__dict__ or key in ['json', '_api']:
            self.__dict__[key] = value
        else:
            self.__dict__['_json'][key] = value

    def __getattr__(self, item):
        if item in self.__dict__:
            return self.__dict__[item]

        if item in self.__dict__['json']:
            return self.__dict__['json'][item]

        raise AttributeError(f'"{item}" does not exist in {type(self)}.')

    def __repr__(self):
        return str(self.json)



from .test_case import TestCaseV09 as TestCase
from sisense.data import Permission
import unittest


class PermissionTestCase(TestCase):

    def test_get(self):
        permission = self.sisense.permission
        permissions = permission.get(self.config['elasticube'])

        self.assertGreaterEqual(len(permissions), 1)

        for obj in permissions:
            self.assertEqual(type(obj), Permission)

        permissions = permission.get('Non existing elasticube')
        self.assertEqual(len(permissions), 1)
        self.assertEqual(permissions[0].permission, None)

    def test_add(self):
        self.sisense.permission.delete_all(self.config['elasticube'])

        self.sisense.permission.delete_all(self.config['elasticube'])
        old_permissions = self.sisense.permission.get(self.config['elasticube'])

        user = self.sisense.user.get(self.config['user_email'])
        self.sisense.permission.add(self.config['elasticube'], user._id, 'user', 'r')

        group = self.sisense.group.get('Dafiti')
        self.sisense.permission.add(self.config['elasticube'], group._id, 'group', 'r')

        new_permissions = self.sisense.permission.get(self.config['elasticube'])
        self.assertEqual(len(old_permissions) + 2, len(new_permissions))

        self.sisense.permission.delete_all(self.config['elasticube'])

    def test_delete_all(self):
        self.sisense.permission.delete_all(self.config['elasticube'])
        new_permissions = self.sisense.permission.get(self.config['elasticube'])

        self.assertEqual(len(new_permissions), 1)


if __name__ == '__main__':
    unittest.main()



from sisense import Sisense
import unittest
import json


def _load_config(version: str) -> dict:
    with open(f'tests/support_files/config/api_{version}.json', 'r') as file:
        return json.load(file)


class TestCaseV09(unittest.TestCase):

    def setUp(self):
        self.config = _load_config('v09')
        self.sisense = Sisense(self.config['host'], self.config['token'])


class TestCaseV1(unittest.TestCase):

    def setUp(self):
        self.config = _load_config('v1')
        self.sisense = Sisense(self.config['host'], self.config['token'])


class TestCaseV2(unittest.TestCase):

    def setUp(self):
        self.config = _load_config('v2')
        self.sisense = Sisense(self.config['host'], self.config['token'])



from sisense.resource import Resource


class User(Resource):

    def get(self, email: str) -> object:
        """Get user by e-mail.

        :param email: (str) User's e-mail.
        :return: User, if found. None, otherwise.
        """
        content = self._api.get('users', query={'email': email})
        return User(self._api, content[0]) if len(content) else None



from .test_case import TestCaseV2 as TestCase
import unittest
import requests
import json
import os


class DatamodelTestCase(TestCase):

    def setUp(self):
        super(DatamodelTestCase, self).setUp()
        self.datamodel = self.sisense.datamodel.get(oid=self.config['datamodel.oid'])

    def test_get_by_id(self):
        self.assertEqual(self.datamodel.title, self.config['datamodel'])

    def test_get_by_name(self):
        datamodel = self.datamodel.get(title=self.config['datamodel'])
        self.assertEqual(datamodel.oid, self.config['datamodel.oid'])

    def test_create(self):
        title = 'SisenseAPI (New) Example'
        new_datamodel = self.datamodel.create(title)
        server_datamodel = self.datamodel.get(title=title)

        self.assertEqual(new_datamodel.oid, server_datamodel.oid)
        server_datamodel.delete()

    def test_clone(self):
        title = 'SisenseAPI (Clone) Example'
        new_datamodel = self.datamodel.clone(title)
        server_datamodel = self.datamodel.get(title=title)

        self.assertEqual(new_datamodel.oid, server_datamodel.oid)
        server_datamodel.delete()

    def test_delete(self):
        new_datamodel = self.datamodel.create('SisenseAPI (Delete) Example')
        oid = new_datamodel.oid
        new_datamodel.delete()

        try:
            server_datamodel = self.datamodel.get(oid=oid)
            self.assertIsNone(server_datamodel.oid)
        except requests.exceptions.HTTPError as error:
            self.assertEqual(error.response.status_code, 404)

    def test_export(self):
        with self.subTest(full=False):
            filename = 'datamodel.smodel'
            self.datamodel.export_datamodel(filename, full=False)

            with open(filename, 'r') as file:
                schema = json.load(file)

            self.assertEqual(schema['title'], self.datamodel.title)

            os.system(f'rm {filename}')

        with self.subTest(full=True):
            filename = 'datamodel.sdata'
            self.datamodel.export_datamodel(filename, full=True)
            new_datamodel = self.datamodel.import_datamodel('SisenseAPI (Import) Example', filename, full=True)

            self.assertEqual(type(new_datamodel.oid), str)
            new_datamodel.delete()

            os.system(f'rm {filename}')

    def test_import(self):
        title = 'SisenseAPI (Import) Example'

        with self.subTest(full=False):
            filename = 'datamodel.smodel'
            self.datamodel.export_datamodel(filename, full=False)
            self.datamodel.import_datamodel(title, filename, full=False)

            new_datamodel = self.datamodel.get(title=title)
            self.assertEqual(new_datamodel.title, title)
            self.assertNotEqual(new_datamodel.oid, self.datamodel.oid)

            new_datamodel.delete()
            os.system(f'rm {filename}')

        with self.subTest(full=True):
            filename = 'datamodel.sdata'
            self.datamodel.export_datamodel(filename, full=True)
            new_datamodel = self.datamodel.import_datamodel(title, filename, full=True)

            self.assertEqual(new_datamodel.title, title)
            self.assertNotEqual(new_datamodel.oid, self.datamodel.oid)

            new_datamodel.delete()
            os.system(f'rm {filename}')


if __name__ == '__main__':
    unittest.main()
